SystemOrganization addCategory: #'CS528su13AES-ValueWithHistory'!

TestCase subclass: #ValueWithHistoryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528su13AES-ValueWithHistory'!

!ValueWithHistoryTest methodsFor: 'testing' stamp: 'm3r 6/27/2008 02:15'!
test
	| history day3 day2 day1 day4 |
	day3 := '12/12/2007' asDate.
	day2 := day3 subtractDays: 3.
	day1 := day2 subtractDays: 3.
	day4 :=  day3 addDays: 4.
	(history := ValueWithHistory new) starting: day3 become: 3.
	history starting: day1 become: 1.
	self should: [(history at: day2) = 1].
	history at: day2 add: 1.
	self should: [(history at: day3) = 4].
	history at: day4 add: 2.
	self should: [(history at: day4) = 6].
! !

Object subclass: #ValueWithHistory
	instanceVariableNames: 'date value'
	classVariableNames: 'EarliestDate'
	poolDictionaries: ''
	category: 'CS528su13AES-ValueWithHistory'!

!ValueWithHistory commentStamp: 'm3r 6/27/2008 01:58' prior: 0!
I represent a (probably numerical) value that changes over time.  I can answer my value at any point in
time (usually a date).  I can update my value at any point in time, and also add a number to my
value from any point in time on into the future.  My value does not change continuously, but changes
at discrete points in time.

Instance Variables:
	date	<SortedCollection of: Date>
	value	<OrderedCollection of: Number>
	The i'th element of value matches the i'th element of date.  The date collection indicates when the value takes on the next element of the value collection.
!

!ValueWithHistory class methodsFor: 'class initialization' stamp: 'm3r 6/27/2008 02:05'!
initialize
	"self initialize"
	EarliestDate := Date newDay: 0 year: 0.
! !

!ValueWithHistory class methodsFor: 'instance creation' stamp: 'm3r 6/27/2008 02:06'!
new
	^ super new initialize
! !

!ValueWithHistory class methodsFor: 'instance creation' stamp: 'm3r 6/27/2008 02:06'!
zero
	^ self new starting: EarliestDate become: 0
! !

!ValueWithHistory methodsFor: 'accessing' stamp: 'm3r 6/27/2008 01:58'!
at: aDate 
	"Return value at aDate"
	| index |
	index := date indexOfStartOfIntervalContaining: aDate.
	index = 0 ifTrue: [self error: 'date is too early'].
	^value at: index
! !

!ValueWithHistory methodsFor: 'accessing' stamp: 'm3r 6/27/2008 01:59'!
at: aDate add: amount 
"Add amount to the value starting at aDate."
	| start |
	start := self indexForAccessing: aDate.
	start to: value size do:
		[:each | value at: each put: (value at: each) + amount].
! !

!ValueWithHistory methodsFor: 'accessing' stamp: 'm3r 6/27/2008 02:00'!
indexForAccessing: aDate 
	"Create index of slot for aDate, creating it if necessary."
	| index |
	(index := date indexOfStartOfIntervalContaining: aDate) = 0 ifTrue: 
		[date add: aDate.
		value addFirst: 0.
		^ 1].
	(date at: index) = aDate ifTrue: [^index].
	date add: aDate.
	value add: (value at: index) beforeIndex: index + 1.
	^ index + 1
! !

!ValueWithHistory methodsFor: 'initialize-release' stamp: 'ChaseGeigle 6/15/2013 23:01'!
initialize
	date := SortedSequence new.
	value := OrderedCollection new.
! !

!ValueWithHistory methodsFor: 'printing' stamp: 'm3r 6/27/2008 02:03'!
printOn: aStream 
	date with: value do: 
		[: nextDate : nextValue | 
		aStream 
			nextPut: $(;
			print: nextDate;
			nextPutAll: ' -> ';
			print: nextValue;
			nextPut: $);
			cr].
! !

!ValueWithHistory methodsFor: 'accessing' stamp: 'm3r 6/27/2008 02:00'!
starting: aDate become: anAmount 
	| index |
	date add: aDate.
	index := date indexOfStartOfIntervalContaining: aDate.
	value add: anAmount beforeIndex: index.
! !
ValueWithHistory initialize!
